@@ -689,6 +689,9 @@ loop_rx_exit:
 }
 
 extern int mtp2_msu_only;
+extern int trace_only_diff;
+extern int trace_rx_only;
+extern int trace_tx_only;
 extern wanpipe_hdlc_engine_t *rx_hdlc_eng;  
 wp_trace_output_iface_t hdlc_trace_iface;
 
@@ -724,6 +727,8 @@ static int trace_aft_hdlc_data(wanpipe_hdlc_engine_t *hdlc_eng, void *data, int
 	return 0;	
 }
 
+static int previous_trace_len=0;
+static int previous_trace_data[5000];
 
 static void line_trace(int trace_mode) 
 {
@@ -840,6 +845,12 @@ static void line_trace(int trace_mode)
 				hdlc_trace_iface.data = trace_iface.data;
 				hdlc_trace_iface.len = trace_iface.len;
 
+				if (trace_rx_only && (trace_iface.status & WP_TRACE_OUTGOING)) {
+					continue;
+				}
+				if (trace_tx_only && !(trace_iface.status & WP_TRACE_OUTGOING)) {
+					continue;
+				}
 
 				/*
 				if (raw_data) {
@@ -852,6 +863,24 @@ static void line_trace(int trace_mode)
 					wanpipe_hdlc_decode(rx_hdlc_eng,trace_iface.data,trace_iface.len);
 					continue;		
 				} 
+	
+				if (mtp2_msu_only) {
+					if (trace_iface.data[2] < 3) {
+						continue;
+					}
+				}			
+
+				if (trace_only_diff) { 
+					if (trace_iface.len == previous_trace_len) {
+						int err=memcmp(trace_iface.data,previous_trace_data,trace_iface.len);
+						if (err == 0) {
+							continue;
+						}	
+					}
+					previous_trace_len=trace_iface.len;
+					memcpy(previous_trace_data,trace_iface.data,trace_iface.len);
+				}
+
 
 			       	if (pcap_output){
 					trace_iface.type=WP_OUT_TRACE_PCAP;
